/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "entityTab.h"

EntityCubeTab::EntityCubeTab(int size, float interval, bool visible,
							 Vector3 axis1, Vector3 axis2, Vector3 axis3, 
							 Vector3 center, Vector3 rotAxis, float angle): 
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), mSize(size), mInterval(interval), mAxis1(axis1), mAxis2(axis2), mAxis3(axis3) {
	
}

EntityCubeTab::EntityCubeTab(int size, float interval, bool visible, Material *material, Mesh *mesh,
							 Vector3 axis1, Vector3 axis2, Vector3 axis3, 
							 Vector3 center, Vector3 rotAxis, float angle): 
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), mSize(size), mInterval(interval), mAxis1(axis1), mAxis2(axis2), mAxis3(axis3) {
		
	for(int i=0;i<mSize;i++) {
		for(int j=0;j<mSize;j++) {
			for(int k=0;k<mSize;k++) {
				addChild(new EntityWithValueInTab (material, mesh, 0, i, j, k, interval, visible, axis1, axis2, axis3), i*mSize*mSize + j*mSize + k);
			}	
		}		
	}
}

EntityCubeTab::~EntityCubeTab() {
}
	

void EntityCubeTab::insertChild(Material *material, Mesh *mesh, int value, int position1, int position2, int position3) {
	int index = position1*mSize*mSize + position2*mSize + position3;
	addChild(new EntityWithValueInTab (material, mesh, value, position1, position2, position3, mInterval, true, mAxis1, mAxis2, mAxis3), index);
}


int EntityCubeTab::getValue(int i, int j, int k) {
	
	int position = i*mSize*mSize + j*mSize + k;
	
	if(children.find(position) != children.end()) {
		EntityWithValueInTab *entity = dynamic_cast <EntityWithValueInTab*> (children[position]);
		return entity->getValue();	
	} else {
		return -1;
	}	
}

int EntityCubeTab::getValue(int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInTab *entity = dynamic_cast <EntityWithValueInTab*> (children[position]);
		return entity->getValue();	
	} else {
		return -1;
	}	
}


bool EntityCubeTab::setValue(int value, int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInTab *entity = dynamic_cast <EntityWithValueInTab*> (children[position]);
		entity->setValue(value);
		return true;
	} else {
		return false;
	}
}

bool EntityCubeTab::setValueAndMaterial(int value, Material *material, int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInTab *entity = dynamic_cast <EntityWithValueInTab*> (children[position]);
		entity->setValue(value);
		entity->setMaterial(material);
		return true;
	} else {
		return false;
	}
}

void EntityCubeTab::setValueAndMaterialForAllEntities(int value, Material *material) {
	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		EntityWithValueInTab *entity = dynamic_cast <EntityWithValueInTab*> ((*it).second);
		entity->setValue(value);
		entity->setMaterial(material);
	}
}

void EntityCubeTab::setClickable() {

	mNumClick = -1;

	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		Entity *child = (*it).second;
		int index = (*it).first;
		child->setNumClick(index);
	}	
}


int EntityCubeTab::testAxis(int direction, int position1, int position2, int value){
	int res = 0;
	
	switch(direction) {
		case 1: //X axis
			for(int i=0;i<mSize;i++) {
				if(getValue(i, position1, position2) == value) {
					res++;
				}
			}
			break;
		case 2: //Y axis
			for(int j=0;j<mSize;j++) {
				if(getValue(position2, j, position1) == value) {
					res++;
				}
			}
			break;
		case 3: //Z axis
			for(int k=0;k<mSize;k++) {
				if(getValue(position1, position2, k) == value) {
					res++;
				}
			}
			break;
	}
	return res;
}

int EntityCubeTab::testDiag(int direction, int position, int value){
	int res = 0;
		
	switch(direction) {
		case 1: //plane YZ
			for(int l=0;l<mSize;l++) {
				if(getValue(position, l, l) == value) {				
					res++;
				}
			}	
			break;
		case -1: //plane YZ
			for(int l=0;l<mSize;l++) {
				if(getValue(position, l, mSize-l-1) == value) {								
					res++;
				}
			}
			break;
		
		case 2: //plane ZX
			for(int l=0;l<mSize;l++) {
				if(getValue(l, position, l) == value) {				
					res++;
				}
			}				
			break;
		
		case -2: //plane ZX
			for(int l=0;l<mSize;l++) {
				if(getValue(l, position, mSize-l-1) == value) {								
					res++;
				}
			}				
			break;
		case 3: //plane XY
			for(int l=0;l<mSize;l++) {
				if(getValue(l, l, position) == value) {								
					res++;
				}
			}				
			break;
		
		case -3: //plane XY
			for(int l=0;l<mSize;l++) {
				if(getValue(l, mSize-l-1, position) == value) {												
					res++;
				}
			}			
			break;
	}
	return res;	
}

int EntityCubeTab::testDiagCube(int direction, int value){
	int res = 0;
	
	switch(direction) {
		case 1:
			for(int l=0;l<mSize;l++) {
				if(getValue(l, l, l) == value) {																
					res++;
				}
			}			
			break;
				
		case 2:
			for(int l=0;l<mSize;l++) {
				if(getValue(mSize-l-1, l, l) == value) {																				
					res++;
				}
			}			
			break;
				
		case 3:
			for(int l=0;l<mSize;l++) {
				if(getValue(l, l, mSize-l-1) == value) {																								
					res++;
				}
			}			
			break;
		
		case 4:
			for(int l=0;l<mSize;l++) {
				if(getValue(l, mSize-l-1, l) == value) {																								
					res++;
				}
			}			
			break;
	}
	
	return res;	
}




