/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "keyboardOptions.h"

GeneralKeyboardOptions *generalKeyboardOptions = 0;


KeyboardOptions::KeyboardOptions(std::string fileName, KeyMap defaultsKeys[], int nbKeys):
	mFileName(fileName), mNbKeys(nbKeys) {
		
	mDefaultsKeys = new KeyMap[mNbKeys];
		
	for(int i=0;i<mNbKeys;i++) {
		mDefaultsKeys[i] = defaultsKeys[i];
	}
		
	setDefaultParameters();
	
	if(fileExists (mFileName)) {
		readConfigurationFile();
	} 	
}

KeyboardOptions::~KeyboardOptions() {
	delete [] mDefaultsKeys;
}

void KeyboardOptions::setDefaultParameters() {
	
	for(int i=0;i<mNbKeys;i++) {
		keys[mDefaultsKeys[i].name] = mDefaultsKeys[i].key;
	}
}

void KeyboardOptions::readConfigurationFile() {
	std::string line;
	std::ifstream file(mFileName.c_str());

	std::string separators = " \n";	
	std::vector<std::string> words;
		
	while(std::getline(file, line)) {
		if(line[0]!='#') {
			words = split(line, separators);
			if((words.size() >=3)&&(words[0]=="set")) {
				keys[words[1]] = StringToSDLKey(words[2]);
			}
		}
	}	
}

std::string KeyboardOptions::writeParam(std::string name, std::vector<std::string> comments) {
	std::string value = SDLKeyToString(SDLKey(keys[name]));
	std::string title = name;
	return writeParamDefinition (title, value, comments);
}


std::string KeyboardOptions::SDLKeyToString(SDLKey key) {
	
	std::string keyString="";
	
	if(isalnum(key)){
		keyString=key;
	}else{
		for(int i=0;i<sizeof(keymap)/sizeof(KeyMap);i++){
			if(key==keymap[i].key){
				keyString=keymap[i].name;
				break;
			}
		}
	}
	return keyString;	
}

SDLKey KeyboardOptions::StringToSDLKey(std::string keyString) {

	if(keyString!=""){
		for(int i=0;i<sizeof(keymap)/sizeof(KeyMap);i++){
			if(keyString==keymap[i].name){
				return keymap[i].key;
			}
		}
		return SDLKey(keyString.c_str()[0]);		
	}else{
		return SDLKey(0);
	}
}


SDLKey KeyboardOptions::getKey(std::string name) {
	if(keys.find(name) != keys.end()) {
		return SDLKey(keys[name]);
	} else {
		return SDLK_UNKNOWN;
	}
}



/********************************/
/*	General keyboard options	*/
/********************************/

GeneralKeyboardOptions::GeneralKeyboardOptions() :
	KeyboardOptions (fileMana->getHomeNewtDir() + "/generalKeyboardOptions.dat",
		(KeyMap[]){{SDLK_ESCAPE,"quit_key"},{SDLK_p,"pause_key"},{SDLK_EQUALS,"screenshot_key"}}, 3){
}

GeneralKeyboardOptions::~GeneralKeyboardOptions() {
}


void GeneralKeyboardOptions::writeConfigurationFile() {
	std::ofstream file(mFileName.c_str(), std::ios::trunc);
	
	if ( !file ){
		std::cerr << "Cannot create general keyboard configuration file." << std::endl;
		exit(1);
	}
	
	std::vector<std::string> comments;
	
	file << "## Newt-Puzzle general keyboard options file. \n";
	file << "# These keys are used in every game or menu. \n \n";
	
	comments.push_back("Key binding for quitting the current game, menu, etc.");
	comments.push_back("If the current state is the game chooser, quit Newt-Puzzle.");
	file << writeParam("quit_key", comments);
	comments.clear();
	
	comments.push_back("Key binding for pausing the game.");
	file << writeParam("pause_key", comments);
	comments.clear();	
	
	comments.push_back("Key binding for taking a screenshot.");
	file << writeParam("screenshot_key", comments);
	comments.clear();
	
	file.close();
	
}

