/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "color.h"

const Color Color::WHITE(1., 1., 1., 1.);
const Color Color::BLACK(0., 0., 0., 0.);


 Color::Color() : r(0.), g(0.), b(0.), a(0.) {
 }

Color::Color(float red, float green, float blue, float alpha):
	r(red), g(green), b(blue), a(alpha) {
}

 Color::Color(int numColor, float coeff, float alpha) : a(alpha){
	setNumColor (numColor);
	applyCoeff(coeff);	 
 }

Color::~Color() {
}

	
void Color::setNumColor(int numColor) {
	switch(numColor) {				
		case 1:
			r = 1.;
			g = 0.;
			b = 0.;
			break;				

		case 2:
			r = 1.;
			g = 0.5;
			b = 0.;
			break;
				
		case 3:
			r = 1.;
			g = 1.;
			b = 0.;		
			break;
				
		case 4:
			r = 0.5;
			g = 1.;
			b = 0.;		
			break;
				
		case 5:
			r = 0.;
			g = 1.;
			b = 0.;		
			break;
				
		case 6:
			r = 0.;
			g = 1.;
			b = 0.5;		
			break;
				
		case 7:
			r = 0.;
			g = 1.;
			b = 1.;		
			break;	
				
		case 8:
			r = 0.;
			g = 0.5;
			b = 1.;		
			break;
				
		case 9:
			r = 0.;
			g = 0.;
			b = 1.;		
			break;
				
		case 10:
			r = 0.5;
			g = 0.;
			b = 1.;			
			break;				

		case 11:
			r = 1.;
			g = 0.;
			b = 1.;			
			break;

		case 12:
			r = 1.;
			g = 0.;
			b = 0.5;			
			break;				

		case 13:
			r = 0.;
			g = 0.;
			b = 0.;			
			break;	

		case 14:
			r = 1.;
			g = 1.;
			b = 1.;			
			break;									
	}	
}

void Color::applyCoeff(float coeff) {
	if(coeff > 0) {
		r = r + (1.-r)*coeff;
		g = g + (1.-g)*coeff;
		b = b + (1.-b)*coeff;
	} else {
		if(coeff < 0) {
			r *= 1.+coeff;
			g *= 1.+coeff;
			b *= 1.+coeff;			
		}
	}	
}
