/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "graphicOptions.h"

GeneralGraphicOptions *generalGraphics = 0;



GeneralGraphicOptions::GeneralGraphicOptions() {
	mFileName = fileMana->getHomeNewtDir() + "/generalGraphicOptions.dat";

	/* default parameters	*/
	currentParameters["screenWidth"] = 800;
	currentParameters["screenHeight"] = 600;
	currentParameters["bppMode"] = 0;
	currentParameters["fullScreen"] = 0;
	currentParameters["doubleBuffer"] = 1;
	currentParameters["useSystemAntiAliasingParam"] = 1;
	currentParameters["nbAntiAliasing"] = 0;
	currentParameters["useSystemAnisotropyParam"] = 1;
	currentParameters["nbAnisotropy"] = 0;
	
	if(fileExists (mFileName)) {
		readConfigurationFile();
	}
	
	nextSessionParameters["screenWidth"] = currentParameters["screenWidth"];
	nextSessionParameters["screenHeight"] = currentParameters["screenHeight"];
	nextSessionParameters["bppMode"] = currentParameters["bppMode"];
	nextSessionParameters["fullScreen"] = currentParameters["fullScreen"];
	nextSessionParameters["doubleBuffer"] = currentParameters["doubleBuffer"];
	nextSessionParameters["useSystemAntiAliasingParam"] = currentParameters["useSystemAntiAliasingParam"];
	nextSessionParameters["nbAntiAliasing"] = currentParameters["nbAntiAliasing"];
	nextSessionParameters["useSystemAnisotropyParam"] = currentParameters["useSystemAnisotropyParam"];
	nextSessionParameters["nbAnisotropy"] = currentParameters["nbAnisotropy"];		
}

GeneralGraphicOptions::~GeneralGraphicOptions() {
}


void GeneralGraphicOptions::readConfigurationFile() {
	std::string line;
	std::ifstream file(mFileName.c_str());

	std::string separators = " \n";	
	std::vector<std::string> words;
		
	while(std::getline(file, line)) {
		if(line[0]!='#') {
			words = split(line, separators);
			if((words.size()>=3)&&(words[0]=="set")) {
				std::istringstream iss(words[2]);
				int value;
				iss >> value; 
				currentParameters[words[1]] = value;
			}
		}
	}
}
	

void GeneralGraphicOptions::writeConfigurationFile() {
	
	std::ofstream file(mFileName.c_str(), std::ios::trunc);
	
	if ( !file ){
		std::cerr << "Cannot create general graphic configuration file." << std::endl;
		exit(1);
	}
	
	std::vector<std::string> comments;
	
	file << "## Newt-Puzzle general graphic options file. \n \n";
	
	comments.push_back("The width of the Newt-Puzzle window.");	
	file << writeParam("screenWidth", comments);
	comments.clear();
	
	comments.push_back("The height of the Newt-Puzzle window.");	
	file << writeParam("screenHeight", comments);
	comments.clear();
	
	comments.push_back("The number of bits per pixels.");
	comments.push_back("If the value is 0, the current setting of the operating system will be used."); 
	file << writeParam("bppMode", comments);
	comments.clear();
	
	comments.push_back("If true, the game will be running in fullscreen mode.");	
	file << writeParam("fullScreen", comments);
	comments.clear();
	
	comments.push_back("If true, the game will use a front and a back buffer.");	
	file << writeParam("doubleBuffer", comments);
	comments.clear();

	comments.push_back("If the value is 1, the game use the operating system parameters for anti-aliasing.");
	file << writeParam("useSystemAntiAliasingParam", comments);
	comments.clear();	
	
	comments.push_back("The number of samples for anti-aliasing.");	
	comments.push_back("If the value is 0, anti-aliasing will be disabled.");
	comments.push_back("This value will be taken into account only if useSystemAntiAliasingParam is 0.");	
	file << writeParam("nbAntiAliasing", comments);
	comments.clear();	

	comments.push_back("If the value is 1, the game use the operating system parameters for anisotropic filtering.");
	file << writeParam("useSystemAnisotropyParam", comments);
	comments.clear();	
	
	comments.push_back("The number of samples for anisotropic filtering.");	
	comments.push_back("If the value is 0, anisotropic filtering will be disabled.");
	comments.push_back("This value will be taken into account only if mUseSystemAnisotropyParam is 0.");	
	file << writeParam("nbAnisotropy", comments);
	comments.clear();
	
	file.close();
}


int GeneralGraphicOptions::getScreenWidth() {
	return currentParameters["screenWidth"];
}

int GeneralGraphicOptions::getScreenHeight() {
	return currentParameters["screenHeight"];
}
	
int GeneralGraphicOptions::getBppMode() {
	return currentParameters["bppMode"];
}

bool GeneralGraphicOptions::isFullscreen() {
	return currentParameters["fullScreen"];
}

bool GeneralGraphicOptions::isDoubleBuffer() {
	return currentParameters["doubleBuffer"];
}

bool GeneralGraphicOptions::isSystemAntiAliasingParamEnable() {
	return currentParameters["useSystemAntiAliasingParam"];
}

int GeneralGraphicOptions::getNbAntiAliasing() {
	return currentParameters["nbAntiAliasing"];
}

bool GeneralGraphicOptions::isSystemAnisotropyParamEnable() {
	return currentParameters["useSystemAnisotropyParam"];
}

int GeneralGraphicOptions::getNbAnisotropy() {
	return currentParameters["nbAnisotropy"];
}


void GeneralGraphicOptions::setScreenWidth(int width) {
	nextSessionParameters["screenWidht"] = width;
}

void GeneralGraphicOptions::setScreenHeight(int height) {
	nextSessionParameters["screenHeight"] = height;
}

void GeneralGraphicOptions::setBppMode(int bpp) {
	nextSessionParameters["bppMode"] = bpp;
}

void GeneralGraphicOptions::setFullscreen(int fullScreen) {
	nextSessionParameters["fullScreen"] = fullScreen;
}

void GeneralGraphicOptions::setDoubleBuffer(int on) {
	nextSessionParameters["doubleBuffer"] = on;
}

void GeneralGraphicOptions::useSystemAntiAliasingParam(int on) {
	nextSessionParameters["useSystemAntiAliasingParam"] = on;
}

void GeneralGraphicOptions::setNbAntiAliasing(int nbSamples) {
	nextSessionParameters["nbAntiAliasing"] = nbSamples;
}

void GeneralGraphicOptions::useSystemAnisotropyParam(int on) {
	nextSessionParameters["useSystemAnisotropyParam"] = on;
}

void GeneralGraphicOptions::setNbAnisotropy(int nbSamples) {
	nextSessionParameters["nbAnisotropy"] = nbSamples;
}



std::string GeneralGraphicOptions::writeParam(std::string name, std::vector<std::string> comments) {
	int value = nextSessionParameters[name];
	return writeParamDefinition (name, value, comments);
}




/********************/
/*	GraphicOptions	*/
/********************/

GraphicOptions::GraphicOptions(std::string specificConfigurationFile) {
	
	setDefaultConfiguration();
	
	std::string genericFileName = fileMana->getHomeNewtDir() + "/genericGraphicsConfiguration.dat";

	if(fileExists (genericFileName)) {
		readConfigurationFile(genericFileName);
	} 
	
	if(fileExists (specificConfigurationFile)) {
		readConfigurationFile(specificConfigurationFile);
	}
	
	if(!GLSLsupported) {
		mGLSLEnabled = false;
		mNormalMapsEnabled = false;	
		mGlossMapsEnabled = false;
		mEmiMapsEnabled = false;
		mRendering = FIXED_PIPELINE_RENDERING;
	}
	
	//TODO provisoire !
	writeAsGenericConfigurationFile();
}


GraphicOptions::~GraphicOptions () {
}


void GraphicOptions::setDefaultConfiguration() {
	if(GLSLsupported) {
		mGLSLEnabled = true;
		mNormalMapsEnabled = true;
		mRendering = PER_FRAG_LIGHT_RENDERING;
	} else {
		mGLSLEnabled = false;
		mNormalMapsEnabled = false;		
		mRendering = FIXED_PIPELINE_RENDERING;
	}
	
	mGlossMapsEnabled = false;
	mEmiMapsEnabled = false;
	
	mReflectionEnabled = 1;
	
	mMaxLightSources = -1;
	mMeshLevel = 1.3f;
	
	mTexturing = TEXTURE_HIGH;
	mShadow = SHADOW_NONE;
}


void GraphicOptions::writeConfigurationFile (std::string filename) {
	std::ofstream file(filename.c_str(), std::ios::trunc);
	
	if ( !file ){
		std::cerr << "Cannot create generic graphics configuration file." << std::endl;
		exit(1);
	}	
	
	std::vector<std::string> comments;
	
	file << "## Newt-Puzzle generic graphics configuration file. \n";
	file << "# These options are used in every game, except if the game has a specific game file. \n \n";
	
	comments.push_back("If the value is 1, use GLSL shaders, else used fixed OpenGL pipeline.");
	file << writeParamDefinition("GLSLEnabled", mGLSLEnabled, comments);
	comments.clear();
	
	comments.push_back("If the value is 1, enable bump mapping.");
	file << writeParamDefinition("NormalMapsEnabled", mNormalMapsEnabled, comments);
	comments.clear();	

	comments.push_back("If the value is 1, enable gloss maps.");
	file << writeParamDefinition("GlossMapsEnabled", mGlossMapsEnabled, comments);
	comments.clear();	
	
	comments.push_back("If the value is 1, enable emission maps.");
	file << writeParamDefinition("EmiMapsEnabled", mEmiMapsEnabled, comments);
	comments.clear();	

	comments.push_back("If the value is 1, enable reflection.");
	file << writeParamDefinition("ReflectionEnabled", mReflectionEnabled, comments);
	comments.clear();	
	
	comments.push_back("If the value is positive, it is the maximal number of lights.");
	comments.push_back("If the value is negative, the number of lights will only be limited by the number of lights supported by your hardware");
	comments.push_back("(usually 8 light sources).");
	comments.push_back("Light sources are performance consuming, so try a low value if the game is too slow.");
	file << writeParamDefinition("MaxLightSources", mMaxLightSources, comments);
	comments.clear();		

	comments.push_back("The level of detail for meshes.");
	comments.push_back("A hight value makes beautiful results, but don't complain if the game is too slow.");
	file << writeParamDefinition("MeshLevel", mMeshLevel, comments);
	comments.clear();		
	
	comments.push_back("If the value is 0, do not use textures.");
	comments.push_back("If the value is 1, use low resolution textures.");
	comments.push_back("If the value is 2, use hight resolution textures.");	
	file << writeParamDefinition("TextureMode", mTexturing, comments);
	comments.clear();
	
	/*comments.push_back("If the value is 0, do not use shadows.");
	comments.push_back("If the value is 1, use shadow maps.");	
	file << writeParamDefinition("ShadowMode", mShadow, comments);
	comments.clear();*/
	
	comments.push_back("If the value is 0, use fixed OpenGL pipeline.");
	comments.push_back("If the value is 1, use per fragment lighting (needs GLSL).");
	//comments.push_back("If the value is 2, use toon rendering (cell shading) (needs GLSL).");	
	file << writeParamDefinition("RenderingMode", mRendering, comments);
	comments.clear();	

	file.close();
}


void GraphicOptions::writeAsGenericConfigurationFile() {
	std::string genericFileName = fileMana->getHomeNewtDir() + "/genericGraphicsConfiguration.dat";
	writeConfigurationFile(genericFileName);

}

void GraphicOptions::readConfigurationFile (std::string fileName) {
	std::string line;
	std::ifstream file(fileName.c_str());

	std::string separators = " \n";	
	std::vector<std::string> words;
		
	while(std::getline(file, line)) {
		if(line[0]!='#') {
			words = split(line, separators);
			readLine(words);
		}
	}
	
	if((!mGLSLEnabled)||(mRendering == FIXED_PIPELINE_RENDERING)) {
		mNormalMapsEnabled = false;
		mGlossMapsEnabled = false;
		mEmiMapsEnabled = false;
		mGLSLEnabled = false;
		mRendering = FIXED_PIPELINE_RENDERING;
	}
}

void GraphicOptions::readLine(std::vector<std::string> words) {
	
	if((words.size() >=3)&&(words[0]=="set")) {
		if(words[1] == "GLSLEnabled") {
			std::istringstream iss(words[2]);
			iss >> mGLSLEnabled;
			return;
		}
		
		if(words[1] == "NormalMapsEnabled") {
			std::istringstream iss(words[2]);
			iss >> mNormalMapsEnabled;
			return;
		}
		
		if(words[1] == "GlossMapsEnabled") {
			std::istringstream iss(words[2]);
			iss >> mGlossMapsEnabled;
			return;
		}
		
		if(words[1] == "EmiMapsEnabled") {
			std::istringstream iss(words[2]);
			iss >> mEmiMapsEnabled;
			return;
		}

		if(words[1] == "ReflectionEnabled") {
			std::istringstream iss(words[2]);
			iss >> mReflectionEnabled;
			return;
		}		
		
		if(words[1] == "MaxLightSources") {
			std::istringstream iss(words[2]);
			int value;
			iss >> value;
			if(value == 0) {
				mMaxLightSources = -1;
			} else if(value > getMaxLights()) {  
				mMaxLightSources = getMaxLights();
			} else {
				mMaxLightSources = value;
			}
			return;
		}			

		if(words[1] == "MeshLevel") {
			std::istringstream iss(words[2]);
			float value;
			iss >> value;
			mMeshLevel = value;
			return;
		}			
		
		if(words[1] == "TextureMode") {
			std::istringstream iss(words[2]);
			int texturing;
			iss >> texturing;
			mTexturing = (textureMode) texturing;
			return;
		}		
		
		if(words[1] == "ShadowMode") {
			std::istringstream iss(words[2]);
			int shadow;
			iss >> shadow;
			mShadow = (shadowMode) shadow;
			return;
		}			
	
		if(words[1] == "RenderingMode") {
			std::istringstream iss(words[2]);
			int rendering;
			iss >> rendering;
			mRendering = (renderingMode) rendering;
			return;
		}	
		
	}
}


void GraphicOptions::setGLSLEnabled(bool on) {
	mGLSLEnabled = on;
}

void GraphicOptions::setNormalMapsEnabled(bool on) {
	mNormalMapsEnabled = on;
}

void GraphicOptions::setEmiMapsEnabled(bool on) {
	mEmiMapsEnabled = on;
}

void GraphicOptions::setGlossMapsEnabled(bool on) {
	mGlossMapsEnabled = on;
}


void GraphicOptions::setReflectionEnabled(int on) {
	mReflectionEnabled = on;
}


bool GraphicOptions::isGLSLEnabled() {
	return mGLSLEnabled;
}


bool GraphicOptions::areNormalMapsEnabled() {
	return mNormalMapsEnabled;
}

bool GraphicOptions::areGlossMapsEnabled() {
	return mGlossMapsEnabled;
}

bool GraphicOptions::areEmiMapsEnabled() {
	return mEmiMapsEnabled;
}


int GraphicOptions::isReflectionEnabled() {
	return mReflectionEnabled;
}

int GraphicOptions::getMaxLightSources() {
	return mMaxLightSources;
}

float GraphicOptions::getMeshLevel() {
	return mMeshLevel;
}

void GraphicOptions::setTextureMode(textureMode texturing) {
	mTexturing = texturing;	
}

void GraphicOptions::setShadowMode(shadowMode shadow) {
	mShadow = shadow;
}

void GraphicOptions::setRenderingMode (renderingMode rendering) {
	mRendering = rendering;
}


textureMode GraphicOptions::getTextureMode() {
	return mTexturing;
}
	
shadowMode GraphicOptions::getShadowMode() {
	return mShadow;
}

renderingMode GraphicOptions::getRenderingMode() {
	return mRendering;
}





