/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GRAPHICOPTIONS_H_
#define _GRAPHICOPTIONS_H_

#include <fstream>
#include <string> 
#include <iostream>
#include <sstream>
#include <vector>
#include <map>

#include "glUtil.h"
#include "fileManager.h"
#include "fileUtil.h"
#include "stringUtil.h"


enum textureMode {TEXTURE_NONE=0, TEXTURE_LOW=1, TEXTURE_HIGH=2};
enum shadowMode {SHADOW_NONE=0,SHADOW_MAP=1};
enum renderingMode {FIXED_PIPELINE_RENDERING=0, PER_FRAG_LIGHT_RENDERING=1, TOON_RENDERING=2};



/*	if we want to change theese options, we need to restart the 3D engine
 	(except for anisotropic parameters)	*/
class GeneralGraphicOptions
{
public:
	GeneralGraphicOptions();
	~GeneralGraphicOptions();

	void readConfigurationFile();
	void writeConfigurationFile();
	
	int getScreenWidth();
	int getScreenHeight();
	int getBppMode();
	bool isFullscreen();
	bool isDoubleBuffer();
	bool isSystemAntiAliasingParamEnable();
	int getNbAntiAliasing();
	bool isSystemAnisotropyParamEnable();
	int getNbAnisotropy();
	
	void setScreenWidth(int width);
	void setScreenHeight(int height);
	void setBppMode(int bpp);
	void setFullscreen(int fullScreen);
	void setDoubleBuffer(int on);
	void useSystemAntiAliasingParam(int on);
	void setNbAntiAliasing(int nbSamples);
	void useSystemAnisotropyParam(int on);
	void setNbAnisotropy(int nbSamples);		
		
protected:
	std::string mFileName;
	std::map<std::string, int> currentParameters;
	std::map<std::string, int> nextSessionParameters;
		
	std::string writeParam(std::string name, std::vector<std::string> comments) ;
		
};

/*	more specific options, can be different for each game	*/
class GraphicOptions
{
public:
	GraphicOptions(std::string specificConfigurationFile);
	virtual ~GraphicOptions ();
		
	void setDefaultConfiguration();
		
	void writeConfigurationFile (std::string filename);
	void writeAsGenericConfigurationFile();
		
	void readConfigurationFile (std::string fileName);
	void readLine(std::vector<std::string> words);
		
	void setGLSLEnabled(bool on);
		
	void setNormalMapsEnabled(bool on);
	void setGlossMapsEnabled(bool on);
	void setEmiMapsEnabled(bool on);
		
	void setReflectionEnabled(int on);
		
	bool isGLSLEnabled();
		
	bool areNormalMapsEnabled();
	bool areGlossMapsEnabled();
	bool areEmiMapsEnabled();
		
	int isReflectionEnabled();
		
	int getMaxLightSources();
	float getMeshLevel();
		
	void setTextureMode(textureMode texturing);
	void setShadowMode(shadowMode shadow);
	void setRenderingMode (renderingMode rendering);
		
	textureMode getTextureMode();
	shadowMode getShadowMode();
	renderingMode getRenderingMode();


		
protected:
	bool mGLSLEnabled;
	
	bool mNormalMapsEnabled;
	bool mGlossMapsEnabled;
	bool mEmiMapsEnabled;
		
	int mReflectionEnabled;
	
	int mMaxLightSources;
	float mMeshLevel;
		
	textureMode mTexturing;
	shadowMode mShadow;
	renderingMode mRendering;
};

extern GeneralGraphicOptions *generalGraphics;

#endif // _GRAPHICOPTIONS_H_
