/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NEWTSTATE_H_
#define _NEWTSTATE_H_

#include <string>
#include <SDL/SDL.h>



#include "./newtEngine/scene.h"

#include "keyboardOptions.h"




class NewtState
{
public:
	NewtState(std::string name, std::string dirName);
	virtual ~NewtState();
	
	std::string getName();
	KeyboardOptions* getKeyboardOptions();
	
	bool haveToStopWastingTime();
		
	void init();
		
	virtual void initDisplayEnvironments () = 0;
	virtual void createObjects () = 0;
	virtual void createLights () = 0;
		
	virtual void updateAnimations(double currentTime, double intervalTime) = 0;
		
	virtual void quit(void) = 0;
	
	//keyboard	
	//return true if the key is repeatable
	virtual bool keyboardEvent(SDLKey key, double timeInterval) = 0;
		
	//mouse event
	virtual void mouseMotion(int xOld, int yOld, int xNew, int yNew, bool left, bool middle, bool right) = 0;
	virtual void leftClick(int x, int y) = 0;
	virtual void rightClick(int x, int y) = 0;
	virtual void middleClick(int x, int y) = 0;
	virtual void mouseWheel(int w) = 0;		
		
		
protected:
	std::string mName;
	std::string mDirPath;
			
	bool mQuit;
		
		
	KeyboardOptions *mKeyboardOptions;
		
};

#endif // _NEWTSTATE_H_
