/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prism.h"
#include "../newtEngine/scene.h"


Prism::Prism(int nbSides, float height, int nbH, float innerRadius, int nbR, float edgeRadius, int nbS,
			 float baseTexOffset, baseType base1, baseType base2, bool inside, 
			 Material *materialSides, Material *materialBase1, Material *materialBase2):
	Entity(NULL, NULL, false), mNbSides(nbSides), mHeight(height), mNbH(nbH), mInnerRadius(innerRadius), mEdgeRadius(edgeRadius), mNbS(nbS)
{
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;

	value = (float) nbR * meshLevel;
	if(value < 2) {
		mNbR = 2;
	} else {
		mNbR = (int) value;
	}	
			
	PolygonMesh *polygon;
	
	
	float edgeLength = edgeRadius * M_PI /2.; 
	float texIntervalH = edgeLength / (edgeLength + height);
	
	edgeLength = edgeRadius * M_PI * (1.-1./nbSides); 
	float rectangleLength = innerRadius * tan(M_PI/nbSides) * 2.;
	float texIntervalS = edgeLength / (edgeLength + rectangleLength);			
	
	
	addChild(new EntityWithPosition(materialSides, new PrismMesh(nbSides, height, nbH, innerRadius, nbS, edgeRadius, nbR, baseTexOffset, base1, base2, inside), true,
									Vector3(0.,0.,0.), Vector3(0.,1.,0.), 0.), 0);
									
	switch(base1) {
		case BASE_FLAT:
			polygon = new PolygonMesh(innerRadius - edgeRadius, nbSides, nbS, baseTexOffset);
			addChild(new EntityWithPosition(materialBase2, polygon, !inside, Vector3(0.,-height/2. - edgeRadius,0.), Vector3(1.,0.,0.), 90.), 1);		
			break;
			
		case BASE_FLAT2:
			polygon = new PolygonMesh(innerRadius - edgeRadius, nbSides, nbS, baseTexOffset);
			//addChild(new EntityWithPosition(materialBase2, polygon, !inside, Vector3(0.,-height/2. - edgeRadius/2.,0.), Vector3(1.,0.,0.), 90.), 1);		
			break;
	}
			
	switch(base2) {
		case BASE_FLAT:
			polygon = new PolygonMesh(innerRadius - edgeRadius, nbSides, nbS, baseTexOffset);
			addChild(new EntityWithPosition(materialBase2, polygon, !inside, Vector3(0.,height/2. + edgeRadius,0.), Vector3(1.,0.,0.), -90.), 2);
			break;
			
		case BASE_FLAT2:
			polygon = new PolygonMesh(innerRadius - edgeRadius, nbSides, nbS, baseTexOffset);
			addChild(new EntityWithPosition(materialBase2, polygon, !inside, Vector3(0.,height/2. + edgeRadius/2.,0.), Vector3(1.,0.,0.), -90.), 2);		
			break;
	}
}

Prism::~Prism() {
	
}


void Prism::setMaterialSides(Material *material) {
	children[0]->setMaterial(material);
}

void Prism::setMaterialBase(Material *material, int base) {
	if((base==1)||(base==2)) {
		children[base]->setMaterial(material);
	}
}

PrismWithPosition::	PrismWithPosition(int nbSides, float height, int nbH, 
									  float innerRadius, int nbS, float edgeRadius, int nbR, 
									  float baseTexOffset, baseType base1, baseType base2, bool inside,
		 	 						  Material *materialSides, Material *materialBase1, Material *materialBase2):
	EntityWithPosition(),
	Prism(nbSides, height, nbH, innerRadius, nbR, edgeRadius, nbS, baseTexOffset, base1, base2, inside, materialSides, materialBase1, materialBase2)
	{
}

PrismWithPosition::~PrismWithPosition() {	
}

/*void PrismWithPosition::applyTransformation() {
	EntityWithPosition::applyTransformation();
}*/


/************************/
/*	PrismMesh			*/
/************************/


PrismMesh::PrismMesh(int nbSides, float height, int nbH, float innerRadius, int nbS, float edgeRadius, int nbR,
					 float baseTexOffset, baseType base1, baseType base2, bool inside) :
Mesh("prism", false, true, false, true), 
mNbSides(nbSides), mHeight(height), mNbH(nbH), mInnerRadius(innerRadius), mNbR(nbR), mEdgeRadius(edgeRadius), mNbS(nbS),
mInside(inside), mBase1(base1), mBase2(base2) {
	
}
	
PrismMesh::~PrismMesh() {
	
}


void PrismMesh::initFirstParams() {
	
	geometry.nbVertices = (mNbH + 1) * ( mNbS + mNbR) * mNbSides;
	geometry.nbIndexArrays = mNbH ;
	
	switch(mBase1) {
		case BASE_FLAT:
			geometry.nbVertices += mNbR * ( mNbS + mNbR) * mNbSides;
			geometry.nbIndexArrays += mNbR;		
			break;
		
		case BASE_FLAT2:
			geometry.nbVertices += mNbR * ( mNbS + mNbR) * mNbSides + 2 * mNbS * mNbSides;
			geometry.nbIndexArrays += mNbR+1;			
			break;
		
	}
	
	switch(mBase2) {
		case BASE_FLAT:
			geometry.nbVertices += mNbR * ( mNbS + mNbR) * mNbSides;
			geometry.nbIndexArrays += mNbR;			
			break;
		
		
		case BASE_FLAT2:
			geometry.nbVertices += mNbR* ( mNbS + mNbR) * mNbSides + 2 * mNbS * mNbSides;
			geometry.nbIndexArrays += mNbR+1;				
			break;
	}	
	
	geometry.primType = new GLenum[geometry.nbIndexArrays];
	geometry.nbIndices = new int[geometry.nbIndexArrays];
	
	
	for(int i=0;i<geometry.nbIndexArrays;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	
	for(int i=0;i<geometry.nbIndexArrays;i++) {
		geometry.nbIndices[i] = 2 * (( mNbS + mNbR) * mNbSides + 1);
	}	
	
	switch(mBase1) {
		case BASE_FLAT:
			geometry.primType[0] = GL_TRIANGLES;
			geometry.nbIndices[0] = 3*(2*mNbS + mNbR) * mNbSides;
			break;
		
		case BASE_FLAT2:
			geometry.nbIndices[0] = 2 * (mNbS * mNbSides + 1);			
			geometry.primType[1] = GL_TRIANGLES;
			geometry.nbIndices[1] = 3*(2*mNbS + mNbR) * mNbSides;		
			break;
	}
	
	switch(mBase2) {
		case BASE_FLAT:
			geometry.primType[geometry.nbIndexArrays-1] = GL_TRIANGLES;
			geometry.nbIndices[geometry.nbIndexArrays-1] = 3*(2*mNbS + mNbR) * mNbSides;		
			break;
		
		case BASE_FLAT2:
			geometry.nbIndices[geometry.nbIndexArrays-1] = 2 * (mNbS * mNbSides + 1);			
			geometry.primType[geometry.nbIndexArrays-2] = GL_TRIANGLES;
			geometry.nbIndices[geometry.nbIndexArrays-2] = 3*(2*mNbS + mNbR) * mNbSides;				
			break;
	}
	
}


void PrismMesh::createVertices(Vector3 *ptr) {
	
	float delta = M_PI / (float) mNbSides;
	float sideLength = 2. * mInnerRadius * tan(delta);
	
	if(mBase1 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								0.,//-mHeight /2. - mEdgeRadius/2., 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								0.,//-mHeight /2. - mEdgeRadius/2., 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								-mHeight /2. - mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								-mHeight /2. - mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
		}
	}
	
	if((mBase1 == BASE_FLAT2)||(mBase1 == BASE_FLAT)) {
		
		//case lambda = -pi/2
		for(int k=0;k<mNbSides;k++) {
			
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								-mHeight /2. - mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								-mHeight /2. - mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
			
			for(int i=0;i<mNbR;i++) {
				*ptr = b;
				*ptr++;
			
			}
		}		
		
		for(int j=-mNbR+1;j<0;j++) {
			
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;
			float sideLengthBis = 2. * (mInnerRadius-mEdgeRadius*cos(lambda)) * tan(delta);

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				Vector3 a = Vector3((mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * sin(alpha-delta), 
									-mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * cos(alpha-delta));
				
				Vector3 b = Vector3((mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * sin(alpha+delta), 
									-mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * cos(alpha+delta));
									
				Vector3 c = Vector3((mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * sin(alpha+delta), 
									-mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * cos(alpha+delta));
				
				for(int i=0;i<mNbS;i++) {
					*ptr = a + (mEdgeRadius*cos(lambda)*tan(delta) + (float) i / (float) mNbS * (sideLengthBis-2.*mEdgeRadius*cos(lambda)*tan(delta)))/sideLengthBis * (b-a);
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
					*ptr = c + Vector3(mEdgeRadius*cos(lambda) * sin(angle), 0., mEdgeRadius*cos(lambda) * cos(angle));
					*ptr++;
				
				}
			}			
		}
	}
	
	for(int j=0;j<=mNbH;j++) {
		for(int k=0;k<mNbSides;k++) {
			
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3(mInnerRadius/cos(delta) * sin(alpha-delta), 
								mHeight * ((float) j / (float) mNbH - 0.5), 
								mInnerRadius/cos(delta) * cos(alpha-delta));
			
			Vector3 b = Vector3(mInnerRadius/cos(delta) * sin(alpha+delta), 
								mHeight * ((float) j / (float) mNbH - 0.5), 
								mInnerRadius/cos(delta) * cos(alpha+delta));
								
			Vector3 c = Vector3((mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * sin(alpha+delta), 
								mHeight * ((float) j / (float) mNbH - 0.5), 
								(mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * cos(alpha+delta));
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + (mEdgeRadius*tan(delta) + (float) i / (float) mNbS * (sideLength-2.*mEdgeRadius*tan(delta)))/sideLength * (b-a);
				*ptr++;
			}
			
			for(int i=0;i<mNbR;i++) {
				float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
				*ptr = c + Vector3(mEdgeRadius * sin(angle), 0., mEdgeRadius * cos(angle));
				*ptr++;
			
			}
		}
	}	

	if((mBase2 == BASE_FLAT2)||(mBase2 == BASE_FLAT)) {
		for(int j=1;j<mNbR;j++) {
			
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;
			float sideLengthBis = 2. * (mInnerRadius-mEdgeRadius*cos(lambda)) * tan(delta);

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				Vector3 a = Vector3((mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * sin(alpha-delta), 
									mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * cos(alpha-delta));
				
				Vector3 b = Vector3((mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * sin(alpha+delta), 
									mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius-mEdgeRadius*(1.-cos(lambda)))/cos(delta) * cos(alpha+delta));
									
				Vector3 c = Vector3((mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * sin(alpha+delta), 
									mHeight /2. + mEdgeRadius * sin(lambda), 
									(mInnerRadius/cos(delta) - mEdgeRadius/cos(delta)) * cos(alpha+delta));
				
				for(int i=0;i<mNbS;i++) {
					*ptr = a + (mEdgeRadius*cos(lambda)*tan(delta) + (float) i / (float) mNbS * (sideLengthBis-2.*mEdgeRadius*cos(lambda)*tan(delta)))/sideLengthBis * (b-a);
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
					*ptr = c + Vector3(mEdgeRadius*cos(lambda) * sin(angle), 0., mEdgeRadius*cos(lambda) * cos(angle));
					*ptr++;
				
				}
			}			
		}
		
		//case lambda = pi/2
		for(int k=0;k<mNbSides;k++) {
			
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								mHeight /2. + mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								mHeight /2. + mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
			
			for(int i=0;i<mNbR;i++) {
				*ptr = b;
				*ptr++;
			
			}
		}				
	}	
	
	
	if(mBase2 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								mHeight /2. + mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								mHeight /2. + mEdgeRadius, 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
			
			Vector3 a = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha-delta), 
								0.,//mHeight /2. + mEdgeRadius/2., 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha-delta));
				
			Vector3 b = Vector3((mInnerRadius-mEdgeRadius)/cos(delta) * sin(alpha+delta), 
								0.,//mHeight /2. + mEdgeRadius/2., 
								(mInnerRadius-mEdgeRadius)/cos(delta) * cos(alpha+delta));
								
			
			for(int i=0;i<mNbS;i++) {
				*ptr = a + ((float) i / (float) mNbS) * (b-a);
				*ptr++;
			}
		}		
	}	
	
}

void PrismMesh::createColors(Color *ptr) {
	
}

void PrismMesh::createNormals(Vector3 *ptr) {
	float delta = M_PI / (float) mNbSides;	
	
	if(mBase1 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;				
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(-sin(alpha), 0., -cos(alpha));
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
						
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(-sin(alpha), 0., -cos(alpha));
				*ptr++;
			}
		}
	}
	
	if((mBase1 == BASE_FLAT2)||(mBase1 == BASE_FLAT)) {
		
		//case lambda = -pi/2
		for(int k=0;k<mNbSides;k++) {								
			for(int i=0;i<mNbS+mNbR;i++) {
				*ptr = Vector3(0., -1., 0.);
				*ptr++;
			}
		}		
		
		for(int j=-mNbR+1;j<0;j++) {
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				for(int i=0;i<mNbS;i++) {
					*ptr = Vector3(sin(alpha)*cos(lambda), sin(lambda), cos(alpha)*cos(lambda));
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
					*ptr = Vector3(sin(angle)*cos(lambda), sin(lambda), cos(angle)*cos(lambda));
					*ptr++;
				}
			}			
		}				
	}
	
	
	for(int j=0;j<=mNbH;j++) {
		for(int k=0;k<mNbSides;k++) {
			
			float alpha = (float) k * delta * 2.;
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(sin(alpha), 0., cos(alpha));
				*ptr++;
			}
			
			for(int i=0;i<mNbR;i++) {
				float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
				*ptr = Vector3(sin(angle), 0., cos(angle));
				*ptr++;			
			}
		}
	}
	

	if((mBase2 == BASE_FLAT2)||(mBase2 == BASE_FLAT)) {
		for(int j=1;j<mNbR;j++) {
			
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				for(int i=0;i<mNbS;i++) {
					*ptr = Vector3(sin(alpha)*cos(lambda), sin(lambda), cos(alpha)*cos(lambda));
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
					*ptr = Vector3(sin(angle)*cos(lambda), sin(lambda), cos(angle)*cos(lambda));
					*ptr++;
				}
			}			
		}
		
		//case lambda = pi/2
		for(int k=0;k<mNbSides;k++) {								
			for(int i=0;i<mNbS+mNbR;i++) {
				*ptr = Vector3(0., 1., 0.);
				*ptr++;
			}
		}				
	}	
	
	
	if(mBase2 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;				
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(-sin(alpha), 0., -cos(alpha));
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
						
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(-sin(alpha), 0., -cos(alpha));
				*ptr++;
			}
		}
	}	
		
	
}

void PrismMesh::createTexCoords(GLfloat *ptr) {
	// TODO !
}

void PrismMesh::createFogCoords(GLfloat *ptr) {
	
}

void PrismMesh::createTangents(Vector3 *ptr) {
	float delta = M_PI / (float) mNbSides;	

	if(mBase1 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;				
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(cos(alpha), 0., -sin(alpha));
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
						
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(cos(alpha), 0., -sin(alpha));
				*ptr++;
			}
		}
	}	
	
	if((mBase1 == BASE_FLAT2)||(mBase1 == BASE_FLAT)) {
		for(int j=-mNbR;j<0;j++) {
			
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				for(int i=0;i<mNbS;i++) {
					*ptr = Vector3(cos(alpha), 0., -sin(alpha));
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * (((float) i +0.5)/ (float) mNbR);
					*ptr = Vector3(cos(angle), 0., -sin(angle));
					*ptr++;
				}
			}			
		}				
	}	
	
	for(int j=0;j<=mNbH;j++) {
		for(int k=0;k<mNbSides;k++) {
			
			float alpha = (float) k * delta * 2.;
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(cos(alpha), 0., -sin(alpha));
				*ptr++;
			}
			
			for(int i=0;i<mNbR;i++) {
				float angle = alpha + 2.*delta * ((float) i / (float) mNbR);
				*ptr = Vector3(cos(angle), 0., -sin(angle));
				*ptr++;
			
			}
		}
	}
	
	if((mBase2 == BASE_FLAT2)||(mBase2 == BASE_FLAT)) {
		for(int j=1;j<=mNbR;j++) {
			
			float lambda = ((float) j / (float) mNbR) * M_PI /2.;

			for(int k=0;k<mNbSides;k++) {
				
				float alpha = (float) k * delta * 2.;
				
				for(int i=0;i<mNbS;i++) {
					*ptr = Vector3(cos(alpha), 0., -sin(alpha));
					*ptr++;
				}
				
				for(int i=0;i<mNbR;i++) {
					float angle = alpha + 2.*delta * (((float) i +0.5)/ (float) mNbR);
					*ptr = Vector3(cos(angle), 0., -sin(angle));
					*ptr++;
				}
			}			
		}				
	}	

	if(mBase2 == BASE_FLAT2) {
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;				
			
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(cos(alpha), 0., -sin(alpha));
				*ptr++;
			}
		}
		for(int k=0;k<mNbSides;k++) {
	
			float alpha = (float) k * delta * 2.;
						
			for(int i=0;i<mNbS;i++) {
				*ptr = Vector3(cos(alpha), 0., -sin(alpha));
				*ptr++;
			}
		}
	}	
	
}
		

void PrismMesh::createIndices(int numArray, GLuint *ptr) {

	int n = ( mNbS + mNbR) * mNbSides;
	int indexOffset = 0;
	int numArrayBis = numArray;
	
	if(mBase1 == BASE_FLAT2) {
		indexOffset = 2*mNbS*mNbSides;
		numArrayBis = numArray-1;
	}
	
	if((mBase1 == BASE_FLAT2)&&(numArray==0)) {
		for(int k=0;k<mNbSides;k++) {
			for(int i=0;i<mNbS;i++) {
				*ptr = mNbS*mNbSides + i+k*mNbSides;	
				ptr++;			
				*ptr = i+k*mNbSides;
				ptr++;			
			}		
		}
		*ptr = mNbS*mNbSides ;
		ptr++;			
		*ptr = 0 ;	
		ptr++;
		
		
	} else if((mBase2 == BASE_FLAT2)&&(numArray==geometry.nbIndexArrays -1)) {
		for(int k=0;k<mNbSides;k++) {
			for(int i=0;i<mNbS;i++) {
				*ptr = indexOffset + numArrayBis*n + mNbS*mNbSides + i+k*mNbSides;	
				ptr++;
				*ptr = indexOffset + numArrayBis*n + i+k*mNbSides;
				ptr++;	
			}			
		}
		*ptr = indexOffset + numArrayBis*n + mNbS*mNbSides;	
		ptr++;
		*ptr = indexOffset + numArrayBis*n ;
		ptr++;		
	
	
	} else if(((mBase1 == BASE_FLAT)&&(numArray==0))||((mBase1 == BASE_FLAT2)&&(numArray==1))){
		int index = 0;
		
		for(int k=0;k<mNbSides;k++) {
			
			for(int i=0;i<mNbS;i++) {
				
				*ptr = indexOffset + numArrayBis*n + index;
				*ptr++;
				*ptr = indexOffset + numArrayBis*n + index+1;
				*ptr++;				
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;
				
				*ptr = indexOffset + numArrayBis*n + index+1;
				*ptr++;	
				*ptr = indexOffset + (numArrayBis+1)*n + index+1;
				*ptr++;				
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;
				
				index ++;
			}
			
			if(k!=mNbSides-1) {
				for(int i=0;i<mNbR;i++) {
					*ptr = indexOffset + numArrayBis*n + index;
					*ptr++;
					*ptr = indexOffset + (numArrayBis+1)*n + index+1;
					*ptr++;	
					*ptr = indexOffset + (numArrayBis+1)*n + index;
					*ptr++;

					index ++;				
				}	
			} else {
				for(int i=0;i<mNbR-1;i++) {
					*ptr = indexOffset + numArrayBis*n + index;
					*ptr++;
					*ptr = indexOffset + (numArrayBis+1)*n + index+1;
					*ptr++;	
					*ptr = indexOffset + (numArrayBis+1)*n + index;
					*ptr++;

					index ++;				
				}
				*ptr = indexOffset + numArrayBis*n + index;
				*ptr++;
				*ptr = indexOffset + (numArrayBis+1)*n;
				*ptr++;	
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;				
				index++;
			}
		}
		
	} else if(((mBase2 == BASE_FLAT)&&(numArray==geometry.nbIndexArrays -1))||((mBase2 == BASE_FLAT2)&&(numArray==geometry.nbIndexArrays -2))){
		
		int index = 0;
		
		for(int k=0;k<mNbSides;k++) {
			
			for(int i=0;i<mNbS;i++) {
				
				*ptr = indexOffset + numArrayBis*n + index;
				*ptr++;
				*ptr = indexOffset + numArrayBis*n + index+1;
				*ptr++;				
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;
				
				*ptr = indexOffset + numArrayBis*n + index+1;
				*ptr++;	
				*ptr = indexOffset + (numArrayBis+1)*n + index+1;
				*ptr++;				
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;
				
				index ++;
			}
			
			if(k!=mNbSides-1) {
				for(int i=0;i<mNbR;i++) {
					*ptr = indexOffset + numArrayBis*n + index;
					*ptr++;
					*ptr = indexOffset + numArrayBis*n + index+1;
					*ptr++;	
					*ptr = indexOffset + (numArrayBis+1)*n + index;
					*ptr++;

					index ++;				
				}	
			} else {
				for(int i=0;i<mNbR-1;i++) {
					*ptr = indexOffset + numArrayBis*n + index;
					*ptr++;
					*ptr = indexOffset + numArrayBis*n + index+1;
					*ptr++;	
					*ptr = indexOffset + (numArrayBis+1)*n + index;
					*ptr++;

					index ++;				
				}
				*ptr = indexOffset + numArrayBis*n + index;
				*ptr++;
				*ptr = indexOffset + numArrayBis*n;
				*ptr++;	
				*ptr = indexOffset + (numArrayBis+1)*n + index;
				*ptr++;				
				index++;
			}
		}
		
		
	} else {
		for(int i=0;i<n;i++) {
		
			if(mInside) {
				*ptr = indexOffset + numArrayBis*n + i;
				ptr++;
				*ptr = indexOffset + (numArrayBis +1)*n + i;	
				ptr++;
			} else {
				*ptr = indexOffset + (numArrayBis +1)*n + i;	
				ptr++;
				*ptr = indexOffset + numArrayBis*n + i;
				ptr++;
			}
		}
		
		if(mInside) {
			*ptr = indexOffset + numArrayBis*n;
			ptr++;		
			*ptr = indexOffset + (numArrayBis +1)*n;
			ptr++;		
		} else {
			*ptr = indexOffset + (numArrayBis +1)*n;
			ptr++;
			*ptr = indexOffset + numArrayBis*n;
			ptr++;
		}	
	}	
}
		

	
	
	
	
	
